@@ -65,7 +65,7 @@ struct mpg123_pars_struct
 	int rva; /* (which) rva to do: 0: nothing, 1: radio/mix/track 2: album/audiophile */
 	long halfspeed;
 	long doublespeed;
-#ifndef WIN32
+#if (!defined (WIN32) || defined (__CYGWIN__))
 	long timeout;
 #endif
 #define NUM_CHANNELS 2
@@ -338,6 +338,7 @@ void frame_gapless_ignore(mpg123_handle *fr, off_t frames);*/
 /* void frame_gapless_buffercheck(mpg123_handle *fr); */
 #endif
 
+/* Number of samples the decoding of the current frame should yield. */
 off_t frame_expect_outsamples(mpg123_handle *fr);
 
 /* Skip this frame... do some fake action to get away without actually decoding it. */
