@@ -13,8 +13,8 @@
 
 #include "getbits.h"
 
-#ifdef WIN32
-#include <winsock.h>
+#if (defined (WIN32) && !defined (__CYGWIN__))
+#include <ws2tcpip.h>
 #endif
 
 /* a limit for number of frames in a track; beyond that unsigned long may not be enough to hold byte addresses */
@@ -569,7 +569,7 @@ init_resync:
 			if((ret=fr->rd->fullread(fr,fr->id3buf+4,124)) < 0){ debug("need more?"); goto read_frame_bad; }
 			fr->metaflags  |= MPG123_NEW_ID3|MPG123_ID3;
 			fr->rdat.flags |= READER_ID3TAG; /* that marks id3v1 */
-			if (VERBOSE2) fprintf(stderr,"Note: Skipped ID3 Tag!\n");
+			if (VERBOSE3) fprintf(stderr,"Note: Skipped ID3v1 tag.\n");
 			goto read_again;
 		}
 		/* duplicated code from above! */
