@@ -37,7 +37,7 @@ struct reader_data
 	off_t filepos; /* position in file or position in buffer chain */
 	int   filept;
 	int   flags;
-#ifndef WIN32
+#if (!defined (WIN32) || defined (__CYGWIN__))
 	long timeout_sec;
 #endif
 	ssize_t (*fdread) (mpg123_handle *, void *, size_t);
@@ -70,7 +70,7 @@ struct reader
 };
 
 /* Open a file by path or use an opened file descriptor. */
-int open_stream(mpg123_handle *, char *path, int fd);
+int open_stream(mpg123_handle *, const char *path, int fd);
 
 /* feed based operation has some specials */
 int open_feed(mpg123_handle *);
