@@ -12,24 +12,18 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
-/* For select(), I need select.h according to POSIX 2001, else: sys/time.h sys/types.h unistd.h */
-/* Including these here although it works without on my Linux install... curious about _why_. */
+/* For select(), I need select.h according to POSIX 2001, else: sys/time.h sys/types.h unistd.h (the latter two included in compat.h already). */
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef _MSC_VER
 #include <io.h>
 #endif
 
+#include "compat.h"
 #include "debug.h"
 
 static int default_init(mpg123_handle *fr);
@@ -64,7 +58,7 @@ static ssize_t plain_read(mpg123_handle *fr, void *buf, size_t count)
 	if(VERBOSE3) debug2("read %li bytes of %li", (long)ret, (long)count);
 	return ret;
 }
-#ifndef WIN32
+#if (!defined (WIN32) || defined (__CYGWIN__))
 
 /* Wait for data becoming available, allowing soft-broken network connection to die
    This is needed for Shoutcast servers that have forgotten about us while connection was temporarily down. */
@@ -234,7 +228,7 @@ static off_t stream_lseek(mpg123_handle *fr, off_t pos, int whence)
 
 static void stream_close(mpg123_handle *fr)
 {
-	if(fr->rdat.flags & READER_FD_OPENED) close(fr->rdat.filept);
+	if(fr->rdat.flags & READER_FD_OPENED) compat_close(fr->rdat.filept);
 	if(fr->rdat.flags & READER_BUFFERED)  bc_reset(&fr->rdat.buffer);
 }
 
@@ -886,7 +880,7 @@ struct reader bad_reader =
 
 static int default_init(mpg123_handle *fr)
 {
-#ifndef WIN32
+#if (!defined (WIN32) || defined (__CYGWIN__))
 	if(fr->p.timeout > 0)
 	{
 		int flags;
@@ -993,6 +987,7 @@ int open_stream(mpg123_handle *fr, const char *bs_filenam, int fd)
 	int filept; /* descriptor of opened file/stream */
 
 	clear_icy(&fr->icy); /* can be done inside frame_clear ...? */
+
 	if(!bs_filenam) /* no file to open, got a descriptor (stdin) */
 	{
 		filept = fd;
@@ -1001,7 +996,7 @@ int open_stream(mpg123_handle *fr, const char *bs_filenam, int fd)
 	#ifndef O_BINARY
 	#define O_BINARY (0)
 	#endif
-	else if((filept = open(bs_filenam, O_RDONLY|O_BINARY)) < 0) /* a plain old file to open... */
+	else if((filept = compat_open(bs_filenam, O_RDONLY|O_BINARY)) < 0) /* a plain old file to open... */
 	{
 		if(NOQUIET) error2("Cannot open file %s: %s", bs_filenam, strerror(errno));
 		fr->err = MPG123_BAD_FILE;
