@@ -17,9 +17,6 @@
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
-#ifdef WIN32
-#include <windows.h>
-#endif
 
 #include <errno.h>
 #include <string.h>
@@ -73,7 +70,10 @@ struct parameter param = {
   0 ,	  /* force_reopen, always (re)opens audio device for next song */
   /* test_cpu flag is valid for multi and 3dnow.. even if 3dnow is built alone; ensure it appears only once */
   FALSE , /* normal operation */
-  FALSE,  /* try to run process in 'realtime mode' */   
+  FALSE,  /* try to run process in 'realtime mode' */
+#ifdef HAVE_WINDOWS_H 
+  0, /* win32 process priority */
+#endif
   NULL,  /* wav,cdr,au Filename */
 	0, /* default is to play all titles in playlist */
 	NULL, /* no playlist per default */
@@ -83,7 +83,7 @@ struct parameter param = {
 #ifdef FIFO
 	,NULL
 #endif
-#ifndef WIN32
+#if !defined (WIN32) || defined (__CYGWIN__)
 	,0 /* timeout */
 #endif
 	,1 /* loop */
@@ -136,6 +136,14 @@ int OutputDescriptor;
 static int filept = -1;
 char *binpath; /* Path to myself. */
 
+/* File-global storage of command line arguments.
+   They may be needed for cleanup after charset conversion. */
+static char **argv = NULL;
+static int    argc = 0;
+
+/* Cleanup marker to know that we intiialized libmpg123 already. */
+static int cleanup_mpg123 = FALSE;
+
 void set_intflag()
 {
 	debug("set_intflag TRUE");
@@ -176,8 +184,14 @@ void safe_exit(int code)
 
 	if(mh != NULL) mpg123_delete(mh);
 
-	mpg123_exit();
+	if(cleanup_mpg123) mpg123_exit();
+
 	httpdata_free(&htd);
+
+#ifdef WIN32_WANT_UNICODE
+	win32_cmdline_free(argc, argv); /* This handles the premature argv == NULL, too. */
+#endif
+
 	/* It's ugly... but let's just fix this still-reachable memory chunk of static char*. */
 	split_dir_file("", &dummy, &dammy);
 	exit(code);
@@ -400,12 +414,15 @@ topt opts[] = {
 	#ifdef NETWORK
 	{'u', "auth",        GLO_ARG | GLO_CHAR, 0, &httpauth,   0},
 	#endif
-	#ifdef HAVE_SCHED_SETSCHEDULER
+	#if defined (HAVE_SCHED_SETSCHEDULER) || defined (HAVE_WINDOWS_H)
 	/* check why this should be a long variable instead of int! */
 	{'T', "realtime",    GLO_LONG,  0, &param.realtime, TRUE },
 	#else
 	{'T', "realtime",    0,  realtime_not_compiled, 0,           0 },    
 	#endif
+	#ifdef HAVE_WINDOWS_H
+	{0, "priority", GLO_ARG | GLO_INT, 0, &param.w32_priority, 0},
+	#endif
 	{0, "title",         GLO_INT,  0, &param.xterm_title, TRUE },
 	{'w', "wav",         GLO_ARG | GLO_CHAR, set_out_wav, 0, 0 },
 	{0, "cdr",           GLO_ARG | GLO_CHAR, set_out_cdr, 0, 0 },
@@ -512,7 +529,7 @@ int open_track(char *fname)
 		filept = http_open(fname, &htd);
 		/* now check if we got sth. and if we got sth. good */
 		if(    (filept >= 0) && (htd.content_type.p != NULL)
-			  && !param.ignore_mime && strcmp(htd.content_type.p, "audio/mpeg") && strcmp(htd.content_type.p, "audio/x-mpeg") )
+			  && !param.ignore_mime && !(debunk_mime(htd.content_type.p) & IS_FILE) )
 		{
 			error1("Unknown mpeg MIME type %s - is it perhaps a playlist (use -@)?", htd.content_type.p == NULL ? "<nil>" : htd.content_type.p);
 			error("If you know the stream is mpeg1/2 audio, then please report this as "PACKAGE_NAME" bug");
@@ -681,7 +698,25 @@ int skip_or_die(struct timeval *start_time)
 #define skip_or_die(a) TRUE
 #endif
 
-int main(int argc, char *argv[])
+#if defined (WANT_WIN32_UNICODE)
+static int
+argv_cleanup(void *in)
+{
+	debug ("argv_cleanup running!\n");
+	char ** ptr;
+	ptr = (char **)in;
+	while (ptr && *ptr) 
+	{
+		free ((void *)*ptr);
+		++ptr;
+	}
+	free(in);
+	debug ("argv_cleanup ran!\n");
+	return 0;
+}
+#endif
+
+int main(int sys_argc, char ** sys_argv)
 {
 	int result;
 	long parr;
@@ -692,6 +727,17 @@ int main(int argc, char *argv[])
 	struct timeval start_time;
 #endif
 
+#if defined (WANT_WIN32_UNICODE)
+	if(win32_cmdline_utf8(&argc, &argv) != 0)
+	{
+		error("Cannot convert command line to UTF8!");
+		safe_exit(76);
+	}
+#else
+	argv = sys_argv;
+	argc = sys_argc;
+#endif
+
 	/* Extract binary and path, take stuff before/after last / or \ . */
 	if((prgName = strrchr(argv[0], '/')) || (prgName = strrchr(argv[0], '\\')))
 	{
@@ -712,13 +758,15 @@ int main(int argc, char *argv[])
 	if(result != MPG123_OK)
 	{
 		error1("Cannot initialize mpg123 library: %s", mpg123_plain_strerror(result));
-		exit(77);
+		safe_exit(77);
 	}
+	cleanup_mpg123 = TRUE;
+
 	mp = mpg123_new_pars(&result); /* This may get leaked on premature exit(), which is mainly a cosmetic issue... */
 	if(mp == NULL)
 	{
 		error1("Crap! Cannot get mpg123 parameters: %s", mpg123_plain_strerror(result));
-		safe_exit(77);
+		safe_exit(78);
 	}
 
 	/* get default values */
@@ -877,7 +925,8 @@ int main(int argc, char *argv[])
 	}
 #endif
 
-#ifdef HAVE_SCHED_SETSCHEDULER
+#if defined (HAVE_SCHED_SETSCHEDULER) && !defined (__CYGWIN__)
+/* Cygwin --realtime seems to fail when accessing network, using win32 set priority instead */
 	if (param.realtime) {  /* Get real-time priority */
 	  struct sched_param sp;
 	  fprintf(stderr,"Getting real-time priority\n");
@@ -888,6 +937,11 @@ int main(int argc, char *argv[])
 	}
 #endif
 
+#ifdef HAVE_WINDOWS_H
+	/* argument "3" is equivalent to realtime priority class */
+	win32_set_priority( param.realtime ? 3 : param.w32_priority);
+#endif
+
 	if(!param.remote) prepare_playlist(argc, argv);
 
 #if !defined(WIN32) && !defined(GENERIC)
@@ -1243,9 +1297,14 @@ static void long_usage(int err)
 	#ifdef HAVE_SETPRIORITY
 	fprintf(o,"        --aggressive       tries to get higher priority (nice)\n");
 	#endif
-	#ifdef HAVE_SCHED_SETSCHEDULER
+	#if defined (HAVE_SCHED_SETSCHEDULER) || defined (HAVE_WINDOWS_H)
 	fprintf(o," -T     --realtime         tries to get realtime priority\n");
 	#endif
+	#ifdef HAVE_WINDOWS_H
+	fprintf(o,"        --priority <n>     use specified process priority\n");
+	fprintf(o,"                           accepts -2 to 3 as integer arguments\n");
+	fprintf(o,"                           -2 as idle, 0 as normal and 3 as realtime.\n");
+	#endif
 	fprintf(o," -?     --help             give compact help\n");
 	fprintf(o,"        --longhelp         give this long help listing\n");
 	fprintf(o,"        --version          give name / version string\n");
