@@ -80,17 +80,19 @@ static size_t rando(size_t n)
 
 char *get_next_file()
 {
-	char *newfile;
+	struct listitem *newitem = NULL;
+
+	if(pl.fill == 0) return NULL;
+
+	++pl.playcount;
 
-	if(pl.fill == 0) newfile = NULL;
-	else
 	/* normal order, just pick next thing */
 	if(param.shuffle < 2)
 	{
 		do
 		{
-			if(pl.pos < pl.fill) newfile = pl.list[pl.pos].url;
-			else newfile = NULL;
+			if(pl.pos < pl.fill) newitem = &pl.list[pl.pos];
+			else newitem = NULL;
 			/* if we have rounds left, decrease loop, else reinit loop because it's a new track */
 			if(pl.loop > 0) --pl.loop; /* loop for current track... */
 			if(pl.loop == 0)
@@ -98,15 +100,26 @@ char *get_next_file()
 				pl.loop = param.loop;
 				++pl.pos;
 			}
-		} while(pl.loop == 0 && newfile != NULL);
+		} while(pl.loop == 0 && newitem != NULL);
 	}
 	else
 	{	/* Randomly select files, with repeating... but keep track of current track for playlist printing. */
-		pl.pos = rando(pl.fill)+1;
-		newfile = pl.list[ pl.pos-1 ].url;
+		do /* limiting randomness: don't repeat too early */
+		{
+			pl.pos = rando(pl.fill);
+		} while( pl.list[pl.pos].playcount && (pl.playcount - pl.list[pl.pos].playcount) <= pl.fill/2 );
+
+		newitem = &pl.list[pl.pos];
 	}
 
-	return newfile; /* "-" is STDOUT, "" is dumb, NULL is nothing */
+	/* "-" is STDOUT, "" is dumb, NULL is nothing */
+	if(newitem != NULL)
+	{
+		/* Remember the playback position of the track. */
+		newitem->playcount = pl.playcount;
+		return newitem->url;
+	}
+	else return NULL;
 }
 
 /* It doesn't really matter on program exit, but anyway...
@@ -137,6 +150,7 @@ void init_playlist()
 	SRAND(time(NULL));
 	pl.file = NULL;
 	pl.entry = 0;
+	pl.playcount = 0;
 	pl.size = 0;
 	pl.fill = 0;
 	pl.pos = 0;
@@ -532,6 +546,7 @@ int add_to_playlist(char* new_entry, char freeit)
 	{
 		pl.list[pl.fill].freeit = freeit;
 		pl.list[pl.fill].url = new_entry;
+		pl.list[pl.fill].playcount = 0;
 		++pl.fill;
 	}
 	else
